#include <WiFi.h>
#include <HTTPClient.h>
#include "esp_camera.h"
#include "esp_http_server.h"
#define CAMERA_MODEL_AI_THINKER // AI-Thinker ESP32-CAM modeli
#include "camera_pins.h"

// Wi-Fi bilgileri
const char* ssid = "";         // Wi-Fi ağ adınız
const char* password = ""; // Wi-Fi şifreniz
// Telegram bot bilgileri
#define BOT_TOKEN ""  
#define CHAT_ID ""  
#define LDR_PIN 12 

// HTTP sunucusu tanımlama
httpd_handle_t server = NULL;

// Son fotoğrafın çekildiği zaman
unsigned long lastCaptureTime = 0;
unsigned long lastLDRCheckTime = 0;  
// Bekleme süresi (30 saniye = 30000 milisaniye)
const unsigned long captureInterval = 30000;
const unsigned long ldrInterval = 1000; 

// Son çekilen fotoğrafı depolamak için pointer
camera_fb_t *lastCapturedFrame = NULL;

bool wifi_durum = false;
bool mesaj_durum = false;

void setup() {
  Serial.begin(115200);  
  pinMode(LDR_PIN, INPUT); 
  analogReadResolution(12); //0-4095
  analogSetAttenuation(ADC_11db); //0V - 3.6V arası
}

void loop() {
  unsigned long currentMillis = millis(); // Mevcut zaman

  // LDR'yi belirli bir aralıkta kontrol et
  if (currentMillis - lastLDRCheckTime >= ldrInterval) {
    int ldrValue = analogRead(LDR_PIN);
    Serial.print("LDR PotiValue: "); 
    Serial.println(ldrValue);
    lastLDRCheckTime = currentMillis;

    if(ldrValue > 350 && wifi_durum == false) {
      // Wi-Fi'ye bağlan
      WiFi.begin(ssid, password);
      Serial.print("Wi-Fi'ye bağlanılıyor");
      while (WiFi.status() != WL_CONNECTED) {
        delay(1000);
        Serial.print(".");
      }
      Serial.println();
      Serial.println("Wi-Fi bağlantısı başarılı!");
      wifi_durum = true;
    }
    
    if(wifi_durum == true && mesaj_durum == false) {
      // Telegram'a mesaj gönder
      sendTelegramMessage("LDR'de değişiklik mevcut!! Kamera Açılıyor...");
      mesaj_durum = true;
      startCamera(); // Kamera başlatma işlemi
    }
  }
}

// Kamera başlatma işlemi
void startCamera() {
  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sccb_sda = SIOD_GPIO_NUM;
  config.pin_sccb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 20000000;
  config.pixel_format = PIXFORMAT_JPEG;   // JPEG formatında çekim
  config.frame_size = FRAMESIZE_VGA;      // 640x480 çözünürlük
  config.jpeg_quality = 12;               // JPEG kalitesi
  config.fb_count = 1;                    // Tek kare buffer

  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK) {
    Serial.printf("Kamera başlatılamadı, hata: 0x%x", err);
    return;
  }
  
  // HTTP sunucusunu başlat
  startServerCam();
  Serial.println("Kamera başlatıldı!");

  // IP adresini seri monitöre yazdır
  String ip_ = WiFi.localIP().toString();
  String yazi = "http://" + ip_;
  sendTelegramMessage(yazi);
}

// HTTP sunucusu başlatma
void startServerCam() {
  httpd_config_t config = HTTPD_DEFAULT_CONFIG();

  // Sunucu başlatılıyor
  if (httpd_start(&server, &config) == ESP_OK) {
    httpd_uri_t capture_uri = {
        .uri       = "/",  // Tarayıcıda ana sayfa (/)
        .method    = HTTP_GET,
        .handler   = capture_handler, // Görüntü yakalama fonksiyonu
        .user_ctx  = NULL
    };
    httpd_register_uri_handler(server, &capture_uri); // Handler'ı kaydet
  } else {
    Serial.println("HTTP sunucusu başlatılamadı.");
  }
}

// Fotoğraf çekme işlevi (capture_handler)
esp_err_t capture_handler(httpd_req_t *req) {
  unsigned long currentTime = millis(); // Şu anki zaman

  // Son çekimden beri geçen süre kontrolü
  if (currentTime - lastCaptureTime >= captureInterval) {
    // Yeni fotoğraf çekiyoruz
    if (lastCapturedFrame != NULL) {
      esp_camera_fb_return(lastCapturedFrame); // Önceki görüntüyü serbest bırak
    }

    lastCapturedFrame = esp_camera_fb_get(); // Kameradan yeni görüntü al

    if (!lastCapturedFrame) {
      Serial.println("Görüntü alınamadı");
      httpd_resp_send_500(req);
      return ESP_FAIL;
    }

    lastCaptureTime = currentTime; // Son çekim zamanını güncelle
  }

  // HTTP yanıtı ayarları (JPEG formatında)
  httpd_resp_set_type(req, "image/jpeg");
  httpd_resp_set_hdr(req, "Content-Disposition", "inline; filename=capture.jpg");

  // Bellekteki son fotoğrafı yanıt olarak gönder
  return httpd_resp_send(req, (const char *)lastCapturedFrame->buf, lastCapturedFrame->len);
}

// Telegram mesajı gönderme
void sendTelegramMessage(String message) {
  HTTPClient http;
  String url = "https://api.telegram.org/bot";
  url += BOT_TOKEN;
  url += "/sendMessage?chat_id=";
  url += CHAT_ID;
  url += "&text=";
  url += message;

  http.begin(url); // URL'yi başlat
  int httpResponseCode = http.GET(); // GET isteği gönder
  
  if (httpResponseCode > 0) {
    String response = http.getString();
    Serial.println("Mesaj gönderildi, Telegram yanıtı:");
    Serial.println(response);
  } else {
    Serial.print("Mesaj gönderilemedi, hata kodu: ");
    Serial.println(httpResponseCode);
  }
  
  http.end(); // HTTP bağlantısını kapat
}
